/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2010 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "griemannHollyModel.H"
#include "mathematicalConstants.H"
#include "fvCFD.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::griemannHollyModel::griemannHollyModel
(
    const Foam::phaseModel& phasea
)
:
    phasea_(phasea),
    Ua_(phasea.U()),
    alpha_(phasea.alpha()),
    phia_(phasea.phi()),
    rhoa_(phasea.rho()),
    da_(phasea.d()),
    nua_(phasea.nu()),

    griemannHollyProperties_
    (
        IOobject
        (
            "griemannHollyProperties",
            Ua_.time().constant(),
            Ua_.mesh(),
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    ),
    griemannHolly_(griemannHollyProperties_.lookup("griemannHolly")),
    turbulentViscosityb_(griemannHollyProperties_.lookup("turbulentViscosityb")),
    e_(griemannHollyProperties_.lookup("e")),
    phi_(dimensionedScalar(griemannHollyProperties_.lookup("phi"))*M_PI/180.0),
    Theta_
    (
        IOobject
        (
            "Theta",
            Ua_.time().timeName(),
            Ua_.mesh(),
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        Ua_.mesh()
    ),
    mua_
    (
        IOobject
        (
            "mua",
            Ua_.time().timeName(),
            Ua_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        Ua_.mesh(),
        dimensionedScalar("zero", dimensionSet(1, -1, -1, 0, 0), 0.0)
    ),
    muf_
    (
        IOobject
        (
            "muf",
            Ua_.time().timeName(),
            Ua_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        Ua_.mesh(),
        dimensionedScalar("zero", dimensionSet(1, -1, -1, 0, 0), 0.0)
    ),
    lambda_
    (
        IOobject
        (
            "lambda",
            Ua_.time().timeName(),
            Ua_.mesh(),
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        Ua_.mesh(),
        dimensionedScalar("zero", dimensionSet(1, -1, -1, 0, 0), 0.0)
    ),
    pa_
    (
        IOobject
        (
            "pa",
            Ua_.time().timeName(),
            Ua_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        Ua_.mesh(),
        dimensionedScalar("zero", dimensionSet(1, -1, -2, 0, 0), 0.0)
    ),
    pf_
    (
        IOobject
        (
            "pf_",
            Ua_.time().timeName(),
            Ua_.mesh(),
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        Ua_.mesh(),
        dimensionedScalar("zero", dimensionSet(1, -1, -2, 0, 0), 0.0)
    ),
    gs0_
    (
        IOobject
        (
            "gs0",
            Ua_.time().timeName(),
            Ua_.mesh(),
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        Ua_.mesh(),
        dimensionedScalar("zero", dimensionSet(0, 0, 0, 0, 0), 1.0)
    )
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::griemannHollyModel::~griemannHollyModel()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

// Update parameters

void Foam::griemannHollyModel::update(const volScalarField& kb, const volScalarField& nutb)
{
    Info << "Updating Griemann and Holly parameters" << endl;

    gs0_ = (2 - alpha_)/(2*pow((1-alpha_),3));

    Theta_ = 2.0/3.0 * kb;

    if(turbulentViscosityb_)
    {
        mua_ = nutb * 4.0/5.0 * alpha_ * rhoa_ * gs0_ * (1+e_) + 4.0/5.0 * alpha_ * rhoa_ * da_ * gs0_ * (1+e_) * sqrt(Theta_/M_PI);
        lambda_ = - 8.0/15.0 * alpha_ * rhoa_ * gs0_ * (1+e_) * nutb + 4.0/5.0 * alpha_ * rhoa_ * da_ * gs0_ * (1+e_) * sqrt(Theta_/M_PI) - 1.0/3.0 * mua_ ;
    }

    else
    {
        mua_ = 4.0/5.0 * alpha_ * rhoa_ * da_ * gs0_ * (1+e_) * sqrt(Theta_/M_PI);
        lambda_ = 2.0/3.0 * mua_;        
    }

    pa_ =  -2.0 * alpha_ * rhoa_ * gs0_ * (1+e_) * Theta_;

}


   
//} 

// ************************************************************************* //
